#!/bin/sh

#  FLAC - Free Lossless Audio Codec
#  Copyright (C) 2001,2002,2003,2004,2005,2006  Josh Coalson
#
#  This file is part the FLAC project.  FLAC is comprised of several
#  components distributed under difference licenses.  The codec libraries
#  are distributed under Xiph.Org's BSD-like license (see the file
#  COPYING.Xiph in this distribution).  All other programs, libraries, and
#  plugins are distributed under the GPL (see COPYING.GPL).  The documentation
#  is distributed under the Gnu FDL (see COPYING.FDL).  Each file in the
#  FLAC distribution contains at the top the terms under which it may be
#  distributed.
#
#  Since this particular file is relevant to all components of FLAC,
#  it may be distributed under the Xiph.Org license, which is the least
#  restrictive of those mentioned above.  See the file COPYING.Xiph in this
#  distribution.

die ()
{
	echo $* 1>&2
	exit 1
}

if [ x = x"$1" ] ; then 
	BUILD=debug
else
	BUILD="$1"
fi

PATH=../../build/bin:$PATH

if [ x"$FLAC__TEST_LEVEL" = x ] ; then
	FLAC__TEST_LEVEL=1
fi

flake -h 1>/dev/null 2>/dev/null || die "ERROR can't find flake executable"

run_flac ()
{
	if [ x"$FLAC__TEST_WITH_VALGRIND" = xyes ] ; then
		echo "valgrind --leak-check=yes --show-reachable=yes --num-callers=100 flake $*" >>test_streams.valgrind.log
		valgrind --leak-check=yes --show-reachable=yes --num-callers=100 --logfile-fd=4 flake $* 4>>test_streams.valgrind.log
	else
		flake $*
	fi
}

echo "Generating streams..."
if [ ! -f wacky1.wav ] ; then
	test_streams || die "ERROR during test_streams"
fi

#
# single-file test routines
#

test_file ()
{
	name=$1
	channels=$2
	bps=$3
	encode_options="$4"

	# convert to wav
	flac -V0f --silent --force-raw-format --endian=little --sign=signed --sample-rate=44100 --bps=$bps --channels=$channels --no-padding --no-seektable $name.raw -c | flac --silent -df -o z.wav - || die "ERROR converting to wav"

	echo -n "$name (--channels=$channels --bps=$bps $encode_options): encode..."
	cmd="run_flac $encode_options z.wav -o $name.flac"
	echo "### ENCODE $name #######################################################" >> ./streams.log
	echo "###    cmd=$cmd" >> ./streams.log
	$cmd 2>>./streams.log || die "ERROR during encode of $name"

	echo -n "decode..."
	cmd="flac --silent --force --endian=little --sign=signed --decode --force-raw-format --output-name=$name.cmp $name.flac"
	echo "### DECODE $name #######################################################" >> ./streams.log
	echo "###    cmd=$cmd" >> ./streams.log
	$cmd 2>>./streams.log || die "ERROR during decode of $name"

	ls -1l $name.raw >> ./streams.log
	ls -1l $name.flac >> ./streams.log
	ls -1l $name.cmp >> ./streams.log

	echo -n "compare..."
	cmp $name.raw $name.cmp || die "ERROR during compare of $name"

	echo OK
}

max_lpc_order=32

echo "Testing small files..."
for ov in '' '-v 1' ; do
	for om in 0 1 2 3 4 5 6 ; do
		test_file test01 1 16 "-12 -m $om $ov -l 1,$max_lpc_order"
		test_file test02 2 16 "-12 -m $om $ov -l 1,$max_lpc_order"
		test_file test03 1 16 "-12 -m $om $ov -l 1,$max_lpc_order"
		test_file test04 2 16 "-12 -m $om $ov -l 1,$max_lpc_order"
	done
done

#@@@for bps in 8 16 24 ; do
for bps in 16 ; do
	echo "Testing 16-bit full-scale deflection streams..."
	for b in 01 02 03 04 05 06 07 ; do
		for ov in '' '-v 1' ; do
			for om in 0 1 2 3 4 5 6 ; do
				test_file fsd$bps-$b 1 $bps "-12 -l 1,$max_lpc_order -m $om $ov"
			done
		done
		for extras in '' '-v 1' ; do
			for opt in 0 1 2 3 4 5 6 7 8 9 10 11 12 ; do
				test_file fsd$bps-$b 1 $bps "-$opt $extras"
			done
		done
		test_file fsd$bps-$b 1 $bps "-t 0"
		test_file fsd$bps-$b 1 $bps "-t 1 -l 0,4"
		for ol in 1 8 12 32 ; do
			for om in 0 1 2 3 4 5 6 ; do
				for or in 0 4 8 ; do
					for blocksize in '-b 32' '-b 1152' '-b 4608' '-b 65535' ; do
						test_file fsd$bps-$b 1 $bps "-3 -l 1,$ol -m $om -s 0 -r 0,$or $blocksize"
					done
				done
			done
		done
	done
done

echo "Testing 16-bit wasted-bits-per-sample streams..."
for b in 01 ; do
	for ov in '' '-v 1' ; do
		for om in 0 1 2 3 4 5 6 ; do
			test_file wbps16-$b 1 16 "-12 -l 1,$max_lpc_order -m $om $ov"
		done
	done
done

#@@@for bps in 8 16 24 ; do
for bps in 16 ; do
	echo "Testing $bps-bit sine wave streams..."
	for b in 00 01 02 03 04 ; do
		for ov in '' '-v 1' ; do
			for om in 0 1 2 3 4 5 6 ; do
				test_file sine${bps}-$b 1 $bps "-12 -l 1,$max_lpc_order -m $om $ov"
				test_file sine${bps}-$b 1 $bps "-12 -l 1,$max_lpc_order -m $om $ov"
				test_file sine${bps}-$b 1 $bps "-12 -l 1,$max_lpc_order -m $om $ov"
			done
		done
	done
	for b in 10 11 12 13 14 15 16 17 18 19 ; do
		for ov in '' '-v 1' ; do
			for om in 0 1 2 3 4 5 6 ; do
				test_file sine${bps}-$b 2 $bps "-12 -l 1,$max_lpc_order -m $om $ov"
				test_file sine${bps}-$b 2 $bps "-12 -l 1,$max_lpc_order -m $om $ov"
				test_file sine${bps}-$b 2 $bps "-12 -l 1,$max_lpc_order -m $om $ov"
			done
		done
	done
done

echo "Testing blocksize variations..."
for blocksize in 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 ; do
	for lpc_order in 1 2 3 4 5 7 8 9 15 16 17 31 32 ; do
		if [ $lpc_order = 0 ] || [ $lpc_order -le $blocksize ] ; then
			test_file sine16-01 1 16 "-12 -l 1,$lpc_order -m 5 -b $blocksize"
		fi
	done
done

echo "Testing some frame header variations..."
test_file sine16-01 1 16 "-12 -l 1,$max_lpc_order -m 5 -b $max_lpc_order"
test_file sine16-01 1 16 "-12 -l 1,$max_lpc_order -m 5 -b 65535"

echo "Testing option variations..."
for f in 00 01 02 03 04 10 11 12 13 14 15 16 17 18 19 ; do
	if [ $f -lt 10 ] ; then
		channels=1
	else
		channels=2
	fi
	for extras in '' '-v 1' ; do
		for opt in 0 1 2 3 4 5 6 7 8 9 10 11 12 ; do
			test_file sine16-$f $channels 16 "-$opt $extras"
		done
	done
	test_file sine16-$f $channels 16 "-t 0"
	test_file sine16-$f $channels 16 "-t 1 -l 0,4"
	for ol in 1 8 12 32 ; do
		for om in 0 1 2 3 4 5 6 ; do
			for os in 0 1 ; do
				if [ $os -eq 0 ] || [ $channels -eq 2 ] ; then
					for or in 0 4 8 ; do
						for blocksize in '-b 32' '-b 1152' '-b 4608' '-b 65535' ; do
							test_file sine16-$f $channels 16 "-3 -l 1,$ol -m $om -s $os -r 0,$or $blocksize"
						done
					done
				fi
			done
		done
	done
done

echo "Testing noise..."
#@@@for channels in 1 2 4 8 ; do
for channels in 1 2 4 ; do
	#@@@for bps in 8 16 24 ; do
	for bps in 16 ; do
		for extras in '' '-v 1' ; do
			for opt in 0 1 2 3 4 5 6 7 8 9 10 11 12 ; do
				test_file noise $channels $bps "-$opt $extras"
			done
		done
		test_file noise $channels $bps "-t 0"
		test_file noise $channels $bps "-t 1 -l 0,4"
		for ol in 1 8 12 32 ; do
			for om in 0 1 2 3 4 5 6 ; do
				for os in 0 1 ; do
					if [ $os -eq 0 ] || [ $channels -eq 2 ] ; then
						for or in 0 4 8 ; do
							for blocksize in '-b 32' '-b 1152' '-b 4608' '-b 65535' ; do
								test_file noise $channels $bps "-3 -l 1,$ol -m $om -s $os -r 0,$or $blocksize"
							done
						done
					fi
				done
			done
		done
	done
done
